@@ -1,5 +1,6 @@
 #include "reader_util.h"
 #include "ndpi_api.h"
+#include "fuzz_common_code.h"
 
 #include <pcap/pcap.h>
 
@@ -17,26 +18,26 @@ u_int8_t enable_protocol_guess = 1, enable_payload_analyzer = 0;
 u_int8_t enable_flow_stats = 0;
 u_int8_t human_readeable_string_len = 5;
 u_int8_t max_num_udp_dissected_pkts = 16 /* 8 is enough for most protocols, Signal requires more */, max_num_tcp_dissected_pkts = 80 /* due to telnet */;
+ndpi_init_prefs init_prefs = ndpi_track_flow_payload | ndpi_enable_ja3_plus;
+int enable_malloc_bins = 0;
+int malloc_size_stats = 0;
+int max_malloc_bins = 0;
+struct ndpi_bin malloc_bins; /* unused */
 
-int bufferToFile(const char * name, const uint8_t *Data, size_t Size) {
-  FILE * fd;
-  if (remove(name) != 0) {
-    if (errno != ENOENT) {
-      perror("remove failed");
-      return -1;
-    }
-  }
-  fd = fopen(name, "wb");
+FILE *bufferToFile(const uint8_t *Data, size_t Size) {
+  FILE *fd;
+  fd = tmpfile();
   if (fd == NULL) {
-    perror("open failed");
-    return -2;
+    perror("Error tmpfile");
+    return NULL;
   }
   if (fwrite (Data, 1, Size, fd) != Size) {
+    perror("Error fwrite");
     fclose(fd);
-    return -3;
+    return NULL;
   }
-  fclose(fd);
-  return 0;
+  rewind(fd);
+  return fd;
 }
 
 int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
@@ -46,8 +47,8 @@ int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
   int r;
   char errbuf[PCAP_ERRBUF_SIZE];
   NDPI_PROTOCOL_BITMASK all;
-  char * pcap_path = tempnam("/tmp", "fuzz-ndpi-reader");
   u_int i;
+  FILE *fd;
 
   if (prefs == NULL) {
     prefs = calloc(sizeof(struct ndpi_workflow_prefs), 1);
@@ -60,10 +61,21 @@ int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
     prefs->max_ndpi_flows = 16 * 1024 * 1024;
     prefs->quiet_mode = 0;
 
+#ifdef ENABLE_MEM_ALLOC_FAILURES
+    fuzz_set_alloc_callbacks();
+#endif
+
     workflow = ndpi_workflow_init(prefs, NULL /* pcap handler will be set later */, 0, ndpi_serialization_format_json);
     // enable all protocols
     NDPI_BITMASK_SET_ALL(all);
     ndpi_set_protocol_detection_bitmask2(workflow->ndpi_struct, &all);
+
+    ndpi_load_protocols_file(workflow->ndpi_struct, "protos.txt");
+    ndpi_load_categories_file(workflow->ndpi_struct, "categories.txt", NULL);
+    ndpi_load_risk_domain_file(workflow->ndpi_struct, "risky_domains.txt");
+    ndpi_load_malicious_ja3_file(workflow->ndpi_struct, "ja3_fingerprints.csv");
+    ndpi_load_malicious_sha1_file(workflow->ndpi_struct, "sha1_fingerprints.csv");
+
     memset(workflow->stats.protocol_counter, 0,
 	   sizeof(workflow->stats.protocol_counter));
     memset(workflow->stats.protocol_counter_bytes, 0,
@@ -73,28 +85,34 @@ int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
     ndpi_finalize_initialization(workflow->ndpi_struct);
   }
 
-  bufferToFile(pcap_path, Data, Size);
+#ifdef ENABLE_MEM_ALLOC_FAILURES
+  /* Don't fail memory allocations until init phase is done */
+  fuzz_set_alloc_seed(Size);
+#endif
 
-  pkts = pcap_open_offline(pcap_path, errbuf);
+  fd = bufferToFile(Data, Size);
+  if (fd == NULL)
+    return 0;
+
+  pkts = pcap_fopen_offline(fd, errbuf);
   if (pkts == NULL) {
-    remove(pcap_path);
-    free(pcap_path);
-    ndpi_term_serializer(&workflow->ndpi_serializer);
+    fclose(fd);
     return 0;
   }
   if (ndpi_is_datalink_supported(pcap_datalink(pkts)) == 0)
   {
     /* Do not fail if the datalink type is not supported (may happen often during fuzzing). */
     pcap_close(pkts);
-    remove(pcap_path);
-    free(pcap_path);
-    ndpi_term_serializer(&workflow->ndpi_serializer);
     return 0;
   }
 
   workflow->pcap_handle = pkts;
   /* Init flow tree */
   workflow->ndpi_flows_root = ndpi_calloc(workflow->prefs.num_roots, sizeof(void *));
+  if(!workflow->ndpi_flows_root) {
+    pcap_close(pkts);
+    return 0;
+  }
 
   header = NULL;
   r = pcap_next_ex(pkts, &header, &pkt);
@@ -119,69 +137,5 @@ int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
     ndpi_tdestroy(workflow->ndpi_flows_root[i], ndpi_flow_info_freer);
   ndpi_free(workflow->ndpi_flows_root);
 
-  remove(pcap_path);
-  free(pcap_path);
-
   return 0;
 }
-
-#ifdef BUILD_MAIN
-int main(int argc, char ** argv)
-{
-  FILE * pcap_file;
-  long pcap_file_size;
-  uint8_t * pcap_buffer;
-  int test_retval;
-
-  if (argc != 2) {
-    fprintf(stderr, "usage: %s: [pcap-file]\n",
-            (argc > 0 ? argv[0] : "fuzz_ndpi_reader_with_main"));
-    return 1;
-  }
-
-  pcap_file = fopen(argv[1], "r");
-  if (pcap_file == NULL) {
-    perror("fopen failed");
-    return 1;
-  }
-
-  if (fseek(pcap_file, 0, SEEK_END) != 0) {
-    perror("fseek(SEEK_END) failed");
-    fclose(pcap_file);
-    return 1;
-  }
-
-  pcap_file_size = ftell(pcap_file);
-  if (pcap_file_size < 0) {
-    perror("ftell failed");
-    fclose(pcap_file);
-    return 1;
-  }
-
-  if (fseek(pcap_file, 0, SEEK_SET) != 0) {
-    perror("fseek(0, SEEK_SET)  failed");
-    fclose(pcap_file);
-    return 1;
-  }
-
-  pcap_buffer = malloc(pcap_file_size);
-  if (pcap_buffer == NULL) {
-    perror("malloc failed");
-    fclose(pcap_file);
-    return 1;
-  }
-
-  if (fread(pcap_buffer, sizeof(*pcap_buffer), pcap_file_size, pcap_file) != (size_t)pcap_file_size) {
-    perror("fread failed");
-    fclose(pcap_file);
-    free(pcap_buffer);
-    return 1;
-  }
-
-  test_retval = LLVMFuzzerTestOneInput(pcap_buffer, pcap_file_size);
-  fclose(pcap_file);
-  free(pcap_buffer);
-
-  return test_retval;
-}
-#endif
