@@ -41,51 +41,21 @@ static int krb_decode_asn1_length(struct ndpi_detection_module_struct *ndpi_stru
                                   size_t * const kasn1_offset)
 {
   struct ndpi_packet_struct * const packet = &ndpi_struct->packet;
-  unsigned char length_octet;
-  int length;
+  int64_t length;
+  u_int16_t value_len;
 
-  length_octet = packet->payload[*kasn1_offset];
+  length = ndpi_asn1_ber_decode_length(&packet->payload[*kasn1_offset],
+				       packet->payload_packet_len - *kasn1_offset,
+				       &value_len);
 
-  if (length_octet == 0xFF)
-  {
-    /* Malformed Packet */
-    return -1;
-  }
-
-  if ((length_octet & 0x80) == 0)
-  {
-    /* Definite, short */
-    length = length_octet & 0x7F;
-    (*kasn1_offset)++;
-  } else {
-    /* Definite, long or indefinite (not support by this implementation) */
-    if ((length_octet & 0x7F) == 0)
-    {
-      /* indefinite, unsupported */
-      return -1;
-    }
-
-    length_octet &= 0x7F;
-    if (length_octet > 4 /* We support only 4 additional length octets. */ ||
-        packet->payload_packet_len <= *kasn1_offset + length_octet + 1)
-    {
-      return -1;
-    }
-
-    int i = 1;
-    length = 0;
-    for (; i <= length_octet; ++i)
-    {
-      length |= (unsigned int)packet->payload[*kasn1_offset + i] << (length_octet - i) * 8;
-    }
-    *kasn1_offset += i;
-  }
-
-  if (packet->payload_packet_len < *kasn1_offset + length)
+  if (length == -1 ||
+      packet->payload_packet_len < *kasn1_offset + value_len + length)
   {
     return -1;
   }
 
+  *kasn1_offset += value_len;
+
   return length;
 }
 
@@ -334,8 +304,8 @@ static void ndpi_int_kerberos_add_connection(struct ndpi_detection_module_struct
 
 /* ************************************************* */
 
-void ndpi_search_kerberos(struct ndpi_detection_module_struct *ndpi_struct,
-			  struct ndpi_flow_struct *flow) {
+static void ndpi_search_kerberos(struct ndpi_detection_module_struct *ndpi_struct,
+				 struct ndpi_flow_struct *flow) {
   struct ndpi_packet_struct *packet = &ndpi_struct->packet;
   u_int16_t sport = packet->tcp ? ntohs(packet->tcp->source) : ntohs(packet->udp->source);
   u_int16_t dport = packet->tcp ? ntohs(packet->tcp->dest) : ntohs(packet->udp->dest);
@@ -424,7 +394,7 @@ void ndpi_search_kerberos(struct ndpi_detection_module_struct *ndpi_struct,
 	
 	return;
       } else if(kerberos_len == expected_len) {
-	if(packet->payload_packet_len > 128) {
+	if(packet->payload_packet_len > 64) {
 	  u_int16_t koffset, i;
 
 	  for(i=8; i<16; i++)
@@ -444,6 +414,7 @@ void ndpi_search_kerberos(struct ndpi_detection_module_struct *ndpi_struct,
 
 	  if(((packet->payload[koffset] == 0x0A)
 	      || (packet->payload[koffset] == 0x0C)
+	      || (packet->payload[koffset] == 0x1E)
 	      || (packet->payload[koffset] == 0x0D)
 	      || (packet->payload[koffset] == 0x0E))) {
 	    u_int16_t koffsetp, body_offset = 0, pad_len;
@@ -597,7 +568,6 @@ void ndpi_search_kerberos(struct ndpi_detection_module_struct *ndpi_struct,
 #ifdef KERBEROS_DEBUG
 	      printf("[Kerberos] Setting extra func from AS-REQ\n");
 #endif
-	      flow->check_extra_packets = 1;
 	      flow->max_extra_packets_to_check = 5; /* Reply may be split into multiple segments */
 	      flow->extra_packets_func = ndpi_search_kerberos_extra;
 	    } else if(msg_type == 0x0e) /* AS-REQ */ {
@@ -653,7 +623,6 @@ void ndpi_search_kerberos(struct ndpi_detection_module_struct *ndpi_struct,
 	      printf("[Kerberos] Setting extra func from TGS-REQ\n");
 #endif
 	      if(!packet->udp) {
-	        flow->check_extra_packets = 1;
 	        flow->max_extra_packets_to_check = 5; /* Reply may be split into multiple segments */
 	        flow->extra_packets_func = ndpi_search_kerberos_extra;
 	      }
@@ -679,6 +648,12 @@ void ndpi_search_kerberos(struct ndpi_detection_module_struct *ndpi_struct,
 	                   sport, dport, flow->protos.kerberos.hostname, flow->protos.kerberos.domain,
 	                   flow->protos.kerberos.username);
 	      flow->extra_packets_func = NULL;
+	    } else if(msg_type == 0x1e) /* Error */ {
+#ifdef KERBEROS_DEBUG
+	      printf("[Kerberos] Processing KRB-Error\n");
+#endif
+	      /* Nothing specific to do; stop dissecting this flow */
+	      flow->extra_packets_func = NULL;
 	    }
 
 	    return;
@@ -718,12 +693,12 @@ static int ndpi_search_kerberos_extra(struct ndpi_detection_module_struct *ndpi_
   ndpi_search_kerberos(ndpi_struct, flow);
 
   /* Possibly more processing */
-  return 1;
+  return flow->extra_packets_func != NULL;
 }
 
 void init_kerberos_dissector(struct ndpi_detection_module_struct *ndpi_struct,
-			     u_int32_t *id, NDPI_PROTOCOL_BITMASK *detection_bitmask) {
-  ndpi_set_bitmask_protocol_detection("Kerberos", ndpi_struct, detection_bitmask, *id,
+			     u_int32_t *id) {
+  ndpi_set_bitmask_protocol_detection("Kerberos", ndpi_struct, *id,
 				      NDPI_PROTOCOL_KERBEROS,
 				      ndpi_search_kerberos,
 				      NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_TCP_OR_UDP_WITH_PAYLOAD_WITHOUT_RETRANSMISSION,
